<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00004000ItemContratoService;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;
use Teknisa\SM\Service\Sm00001000ItemLancamentoPropostaService;
use Teknisa\Libs\Util\NewCode;

class Sm00004000ItemContratoController extends SMCrud {
    protected $dataSourceNameToSave = 'fct_itemcontrato';
    protected $dataSourceName = 'fct_itemcontrato_widget';
    protected $itensService;
    protected $Environment;
    protected $itensPropostaService;
    
    public function __construct ($crudService, Environment $environment, Sm00004000ItemContratoService $itensService, Sm00001000LancamentoPropostaService $propostaService, Sm00001000ItemLancamentoPropostaService $itensPropostaService, NewCode $newCode){
        parent::__construct($crudService, $environment);
        $this->itensService         = $itensService;
        $this->Environment          = $environment;
        $this->propostaService      = $propostaService;
        $this->itensPropostaService = $itensPropostaService;
        $this->newCode              = $newCode;
    }
    
    public function find(Request\Filter $request, Response $response){
        try {                                                                            
            $request->setParameter("dbname", "fct_itemorcamento_arvore_medidas_variaveis_cont");
            parent::find($request, $response);
            $rows = $response->getDataSets()[0]->getRows();
            $rows = $this->formatarValoresGrid($rows);
            $response->setDataSets(array(new DataSet("fct_itemorcamento_arvore_medidas_variaveis_cont", $rows)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
    * efetua os calculos dos valores ligados ao item 
    **/
    public function formatarValoresGrid($rows){
        foreach($rows as &$row){
            if($row["IDTIPO"] == "M"){
                $row["VRUNITITEMCONT_GRID"]  = number_format((double)$row["VRUNITITEMCONT_GRID"], 2, ",", ".");
                $row["QTMEDIDACONTR_GRID"]   = number_format((double)$row["QTMEDIDACONTR_GRID"], 2, ",", ".");
            }
        }
        return $rows;
    }
    /**
     * salvar um item do contrato
    **/
    public function save(Request $request, Response $response) {
        $rows        = $request->getDataSet()->getRows();
        $nrorg      = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $indice = 0;
        
        //preparar para inserir
        foreach($rows as $row){
            if($row["__is_new"])
                $row["NRITEMOCONTRATO"] = $this->newCode->getCodeByTableName("FCT_ITEMCONTRATO", $nrorg, 1, 10, true);
            
            if(!isset($row["QTMEDIDACALCULADA"]))
                $row["QTMEDIDACALCULADA"] = $row["QTMEDIDACONTR"];
        }
        //inserir as rows
        parent::save($request, $response);
        foreach($rows as $row){
            try {
                $columnsToReturn = $response->getDataSets()[0]->getRows()[$indice];
                $nritemocontrato = $columnsToReturn["NRITEMOCONTRATO"];
                if($row["ITEMCOMPOSICAO"]) {
                    if($row["IDTIPO"] == "M") 
                        $this->itensService->replicarAlteracoesParaItensPais($row, $nrorg, $cdoperador);
                    
                    $qtmedidacontrato          = $row["QTMEDIDACONTR"];
                    $row["CDTPMEDIDAVARIAVEL"] = $row["CDTPMEDIDASERV_GRID"];
                    $this->itensService->inserirOuAtualizarMedidaVariavelServico($row, $nrorg, $cdoperador, true);
                    
                    $row["QTMEDIDACONTR"] = $qtmedidacontrato;
                    $this->itensService->atualizarValorItensNegociacaoValores($row["NRSEQCONTRATO"], $row["NRITEMOCONTRATO"], $row["VRTOTITEMCONT"], $row["NRITEMCONTRATOPAI"], $nrorg, $cdoperador);
                    $itensProposta = $this->itensService->getItensPropostaGeradaPorContrato($row);
                    if(count($itensProposta) > 0) {
                        $itensProposta  = $itensProposta[0];
                        $nrseqorcamento = $itensProposta["NRSEQORCAMENTO"];
                        $cdservorca     = $itensProposta["CDSERVORCA"];     
                        
                        $this->itensService->salvarItemComposicaoProposta($row, $itensProposta);
                        $this->itensPropostaService->replicaAteracaoParaItemPai($nrseqorcamento, $nrorg, $cdoperador, $cdservorca);
                        
                        $qtmedidaorca = $this->propostaService->getSomaQuantidadeMedidaServicoProposta($nrseqorcamento, $cdservorca, $nrorg);
                        $itensProposta["QTMEDIDAORCA"] = $qtmedidaorca;
                        $this->propostaService->inserirOuAtualizarMedidaVariavelServico($itensProposta, $nrorg, $cdoperador, true);
                        
                        $nritemorcamentopai = $this->itensPropostaService->getCodigoItemPropostaPai($row["NRITEMORCAMENTO"], $nrorg);
                        $this->itensPropostaService->atualizarValorItensNegociacaoValores($nrseqorcamento, $itensProposta["NRITEMORCAMENTO"], $itensProposta["VRTOTALITEMORCA"], $nritemorcamentopai, $nrorg, $cdoperador);
                        
                        $this->propostaService->atualizarValorProposta($nrseqorcamento, $nrorg);
                    }
                } else {
                    if ($row["__is_new"]) {
                        $this->itensService->inserirItemNegociacaoValoresContrato($row, $nrorg, $cdoperador);
    
                        //$nrtipoorcamento = $row["NRTIPOORCAMENTO"];
                        //$this->itensService->gerarRateioItensManualmente($row, $nrtipoorcamento, $nrorg);
                    }
    
                    if (isset($row["INSERIR_MEDIDA"]) && $row["INSERIR_MEDIDA"] == true)
                        $this->itensService->inserirOuAtualizarMedidaVariavelServico($row, $nrorg, $cdoperador);
                    
                    $this->itensService->excluirMedidasVariaveisContratoSemItemRelacionado($row["NRSEQCONTRATO"], $nrorg);
                }
                
                $indice++;
            } catch(\Exception $e) {
                if ($row["__is_new"] && isset($nritemocontrato)) {
                    $this->itensService->excluirItemContrato($nritemocontrato, $nrorg);
                }
                $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    /**
     *Valida se o percentual máximo de desconto dado pelo o vendedor esta conforme com seu perfil de acesso ou operador
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function validaPorcentualDescontoAcrescimoSave(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $cdservorca = $row["CDSERVORCA"];
            $idtipo = $row["IDTIPO"];

            $nrorg = $this->Environment->getNrOrg();
            
            $cdoperador =$this->Environment->getCdOperador();
            $desconto = $this->itensService->validaPorcentualDescontoAcrescimoSave($cdservorca, $cdoperador, $idtipo, $nrorg);
            $response->addDataSet(new DataSet("RESULTADO", array("VRPERCDESC" => $desconto)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta da tabela de agendamentos pendentes os items deletados no 
     * contrato aprovado
    **/
    public function DeletaItemContratoAprovado(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            $nritemocontrato = $row["NRITEMOCONTRATO"];
            //$nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $nrorg           = $this->Environment->getNrOrg();
            
            $this->itensService->excluiItemAgendPendente($nrseqcontrato, $nritemocontrato, $nrorg);

            $cdservorca = $row["CDSERVORCA"];
            $this->itensService->excluirMedidaVariavelServico($nrseqcontrato, $cdservorca, $nrorg);

            $request->setParameter("dbname", $this->dataSourceName);
            parent::delete($request, $response);
            
            $this->itensService->excluirMedidasVariaveisContratoSemItemRelacionado($nrseqcontrato, $nrorg);
            //$this->itensService->gerarRateioItensManualmente($row, $nrtipoorcamento, $nrorg);
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta da tabela de agendamentos pendentes os items deletados no 
     * contrato aprovado
    **/
    public function atualizaQuantidadeItensAgendPendente(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            $nritemocontrato = $row["NRITEMOCONTRATO"];
            $quantidade      = $row["QTPENDENCIA"];
            $situacao = isset($row["SITUACAO"]) ? $row["SITUACAO"] : null;
            $nrorg           = $this->Environment->getNrOrg();
            $cdoperador      = $this->Environment->getCdOperador();
            
            $this->itensService->atualizaQuantidadeItensAgendPendente($nrseqcontrato, $nritemocontrato, $nrorg, $quantidade, $cdoperador, $situacao);
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta as formações de preço que foram alteradas e não utilizadas
    **/
    public function deletaAlteracoesInutiFormCont(Request\Row $request, Response $response){
        try {
            $row             = $request->getRow();
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            $nritemocontrato = $row["NRITEMOCONTRATO"];
            $nrorg           = $this->environment->getNrOrg();
            
            $this->itensService->deletaAlteracoesInutiFormCont($nrseqcontrato, $nritemocontrato, $nrorg);
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * insere uma alteração da formação de preço
     * durante a inserção dos itens no contrato
    **/
    public function insereAlteracaoFormPrecCont(Request\Row $request, Response $response){
        try {
            
            $row                = $request->getRow();
            $nrorg              = $this->environment->getNrOrg();
            $operador           = $this->environment->getCdOperador();
            $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
            $nrseqcontrato      = $row["NRSEQCONTRATO"];
            $nritemocontrato    = $row["NRITEMOCONTRATO"];
            $novoCodigo = $this->itensService->executarProcedureFctCopiaFormPrecoItCont($nrformprecoservico, $nrorg, $operador, $nrseqcontrato, $nritemocontrato);
            
            /*if (isset($row["NRCALCFORMPRECSERV"])) {
                $nrcalcformprecserv = $row["NRCALCFORMPRECSERV"];
                $this->itensService->excluirItensFormacaoNaoUsados($nrseqcontrato, $nritemocontrato, $nrformprecoservico, $nrcalcformprecserv, $nrorg, $operador);
            }*/
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * insere o número do item gerado durante a inserção da formação alterada
     * no contrato na tabela de formação de preço auxiliar
    **/
    public function editaItensFormacaoAuxCont(Request $request, Response $response){
        try {
            $rows     = $request->getDataSet()->getRows();
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $this->itensService->editaItensFormacaoAuxCont($rows, $nrorg, $operador);
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta o item de formação de preço inserido quando a rotina de alteração do número 
     * do item na tabela de formção de preço auxiliar não funciona
    **/
    public function deletarItensFormacao(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $nrseqcontrato      = $row["NRSEQCONTRATO"];
            $nritemocontrato    = $row["NRITEMOCONTRATO"];
            
            $this->itensService->deletarItensFormacao($nrseqcontrato, $nritemocontrato, $nrorg);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * deleta a árvore completa de elementos alterados na formação de 
     * preço auxiliar já inserida no contrato
     **/
    public function deletarArvoreFormPrecoItCont(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            $nritemocontrato = $row["NRITEMOCONTRATO"];
            $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
            $nrcalcformprec     = $row["NRCALCFORMPRECSERV"];
            $this->itensService->deletarArvoreFormPrecoItCont($nrseqcontrato, $nritemocontrato, $nrorg, $nrformprecoservico, $nrcalcformprec);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Atualiza o valor do item no contrato após alterações nos 
     * elementos ou no tipo de medida
    **/
    public function atualizarItemContComposicao(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $this->itensService->atualizarItemContComposicao($nrorg, $cdoperador, $nrseqcontrato, $nrseqorcamento);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function verificarSeExisteDescontoNegociacaoContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $nrorg = $this->environment->getNrOrg();

            $existeDesconto = $this->itensService->verificarSeExisteDescontoNegociacaoContrato($nrnegociacaocontrato, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("existe" => $existeDesconto)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function verificarSeExisteDescontoItensNegociacaoContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $nrorg = $this->environment->getNrOrg();

            $existeDesconto = $this->itensService->verificarSeExisteDescontoItensNegociacaoContrato($nrnegociacaocontrato, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("existe" => $existeDesconto)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getServicoPercentualMinimoDescontoPermitidoNegociacaoContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $cdoperador = $this->environment->getCdOperador();
            $nrorg = $this->environment->getNrOrg();

            $servico = $this->itensService->getServicoPercentualMinimoDescontoPermitidoNegociacaoContrato($nrnegociacaocontrato, $cdoperador, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("servico" => $servico)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Manda salvar alterações no nome contratual de um contrato
    **/
    public function salvarNomeContratualContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $contrato      = $row["CONTRATO"];
            $idreplicaorca = isset($row["IDREPLICARORCA"]) ? $row["IDREPLICARORCA"] : false;
            $rowAlteradas  = $row["ROWSALTERADAS"];
            $cdoperador    = $this->environment->getCdOperador();
            $nrorg         = $this->environment->getNrOrg();
            
            $this->itensService->salvarNomeContratualContrato($rowAlteradas, $nrorg, $cdoperador);
            if($idreplicaorca) {
                $nmContratuaisOrca = $this->itensService->getInfoNomeContratualOrca($contrato, $nrorg, $rowAlteradas);
                $this->itensPropostaService->salvarNomeContratual($nmContratuaisOrca, $nrorg, $cdoperador);
            }
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Valida se a quantidade de medida estipulada em um item do contrato 
     * é permitida e se a faixa de variação de preço não foi alterada
    **/
    public function validarQuantidadeMedidaItens(Request\Row $request, Response $response) {
        try {
            $row           = $request->getRow();
            $cdoperador    = $this->environment->getCdOperador();
            $nrorg         = $this->environment->getNrOrg();
            $vrFaixaAtual  = $this->itensService->validarVaricaoDePrecoItens($row, $nrorg, $cdoperador);
            
            if($vrFaixaAtual !== null) 
                $response->addDataSet(new DataSet("fct_itemcontrato_widget", $vrFaixaAtual));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * retorna o valor do serviço na tabela de preço 
    **/
    public function getValorUnitarioTblPrecoItemContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $cdservorca      = $row["CDSERVORCA"];
            $cdmedidavar     = $row["CDTPMEDIDAVARIAVEL"];
            $qtdmedida       = $row["QTMEDIDASERV"];
            $nrorg           = $this->environment->getNrOrg();

            $vrpreco = $this->itensService->getValorUnitarioTblPrecoItemContrato($nrtipoorcamento, $cdservorca, $cdmedidavar, $qtdmedida, $nrorg);
            $response->addDataSet(new DataSet("fct_itemcontrato_widget", array("VRPRECO" => $vrpreco)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Atualiza o valor do rateio
    **/
    public function atualizarValorRateiosContrato(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $cdoperador = $this->environment->getCdOperador();
            $nrorg      = $this->environment->getNrOrg();
            
            $nritemocontrato      = $row["NRITEMOCONTRATO"];
            $nrseqcontrato        = $row["NRSEQCONTRATO"];
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $this->itensService->atualizarValorRateiosContrato($nrseqcontrato, $nritemocontrato, $nrnegociacaocontrato, $nrorg, $cdoperador);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function regerarNegociacaoValores(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->itensService->regerarNegociacaoValores($nrseqcontrato, $nrtipoorcamento, $cdoperador, $nrorg);
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
?>